=== modified file 'database/schema/comments.sql'
--- database/schema/comments.sql	2011-06-16 10:51:17 +0000
+++ database/schema/comments.sql	2011-06-19 07:26:23 +0000
@@ -1629,7 +1629,7 @@
 
 COMMENT ON TABLE BinaryPackageReleaseContents IS 'BinaryPackageReleaseContents: Mapping table that maps from BinaryPackageReleases to path names.';
 COMMENT ON COLUMN BinaryPackageReleaseContents.binarypackagerelease IS 'The BinaryPackageRelease that contains the path name.';
-COMMENT ON COLUMN BinaryPackageReleaseContents.binarypackagepath IS 'The path name, via the BinaryPackagePaths table.';
+COMMENT ON COLUMN BinaryPackageReleaseContents.binarypackagepath IS 'The path name, via the BinaryPackagePath table.';
 
 -- BinaryPackageFile
 
@@ -1642,10 +1642,10 @@
 
 COMMENT ON TABLE BinaryPackageName IS 'BinaryPackageName: A soyuz binary package name.';
 
--- BinaryPackagePaths
+-- BinaryPackagePath
 
-COMMENT ON TABLE BinaryPackagePaths IS 'BinaryPackagePaths: A table of filenames shipped in binary packages.';
-COMMENT ON COLUMN BinaryPackagePaths.path IS 'The full path of the file.';
+COMMENT ON TABLE BinaryPackagePath IS 'BinaryPackagePath: A table of filenames shipped in binary packages.';
+COMMENT ON COLUMN BinaryPackagePath.path IS 'The full path of the file.';
 
 -- Distribution
 

=== modified file 'database/schema/patch-2208-76-0.sql'
--- database/schema/patch-2208-76-0.sql	2011-06-16 10:51:17 +0000
+++ database/schema/patch-2208-76-0.sql	2011-06-19 07:26:23 +0000
@@ -3,14 +3,14 @@
 
 SET client_min_messages=ERROR;
 
-CREATE TABLE BinaryPackagePaths (
+CREATE TABLE BinaryPackagePath (
     id serial PRIMARY KEY,
     path text UNIQUE NOT NULL
 );
 
 CREATE TABLE BinaryPackageReleaseContents (
     binarypackagerelease integer NOT NULL REFERENCES BinaryPackageRelease,
-    binarypackagepath integer NOT NULL REFERENCES BinaryPackagePaths,
+    binarypackagepath integer NOT NULL REFERENCES BinaryPackagePath,
     PRIMARY KEY (binarypackagerelease, binarypackagepath)
 );
 

=== modified file 'database/schema/security.cfg'
--- database/schema/security.cfg	2011-06-13 09:43:22 +0000
+++ database/schema/security.cfg	2011-06-19 07:26:23 +0000
@@ -119,7 +119,9 @@
 public.archivesubscriber                = SELECT, INSERT, UPDATE
 public.authtoken                        = SELECT, INSERT, UPDATE, DELETE
 public.binaryandsourcepackagenameview   = SELECT
+public.binarypackagepath                = SELECT, INSERT, DELETE
 public.binarypackagepublishinghistory   = SELECT
+public.binarypackagereleasecontents     = SELECT, INSERT, DELETE
 public.binarypackagereleasedownloadcount= SELECT, INSERT, UPDATE
 public.bountysubscription               = SELECT, INSERT, UPDATE, DELETE
 public.branch                           = SELECT, INSERT, UPDATE, DELETE

=== modified file 'lib/lp/soyuz/configure.zcml'
--- lib/lp/soyuz/configure.zcml	2011-06-15 02:41:34 +0000
+++ lib/lp/soyuz/configure.zcml	2011-06-19 07:26:23 +0000
@@ -982,6 +982,50 @@
         <allow interface="lp.soyuz.adapters.overrides.IOverridePolicy" />
    </class>
 
+    <!-- BinaryPackagePath -->
+
+    <class
+        class="lp.soyuz.model.binarypackagepath.BinaryPackagePath">
+        <allow
+            interface="lp.soyuz.interfaces.binarypackagepath.IBinaryPackagePath"/>
+    </class>
+
+    <!-- BinaryPackagePathSource -->
+
+    <class
+        class="lp.soyuz.model.binarypackagepath.BinaryPackagePath">
+        <allow
+            interface="lp.soyuz.interfaces.binarypackagepath.IBinaryPackagePathSource"/>
+    </class>
+    <securedutility
+        class="lp.soyuz.model.binarypackagepath.BinaryPackagePath"
+        provides="lp.soyuz.interfaces.binarypackagepath.IBinaryPackagePathSource">
+        <allow
+            interface="lp.soyuz.interfaces.binarypackagepath.IBinaryPackagePathSource"/>
+    </securedutility>
+
+    <!-- BinaryPackageReleaseContents -->
+
+    <class
+        class="lp.soyuz.model.binarypackagereleasecontents.BinaryPackageReleaseContents">
+        <allow
+            interface="lp.soyuz.interfaces.binarypackagereleasecontents.IBinaryPackageReleaseContents"/>
+    </class>
+
+    <!-- BinaryPackageReleaseContentsSet -->
+
+    <class
+        class="lp.soyuz.model.binarypackagereleasecontents.BinaryPackageReleaseContents">
+        <allow
+            interface="lp.soyuz.interfaces.binarypackagereleasecontents.IBinaryPackageReleaseContentsSet"/>
+    </class>
+    <securedutility
+        class="lp.soyuz.model.binarypackagereleasecontents.BinaryPackageReleaseContents"
+        provides="lp.soyuz.interfaces.binarypackagereleasecontents.IBinaryPackageReleaseContentsSet">
+        <allow
+            interface="lp.soyuz.interfaces.binarypackagereleasecontents.IBinaryPackageReleaseContentsSet"/>
+    </securedutility>
+
    <webservice:register module="lp.soyuz.interfaces.webservice" />
 
 </configure>

=== added file 'lib/lp/soyuz/interfaces/binarypackagepath.py'
--- lib/lp/soyuz/interfaces/binarypackagepath.py	1970-01-01 00:00:00 +0000
+++ lib/lp/soyuz/interfaces/binarypackagepath.py	2011-06-19 07:26:23 +0000
@@ -0,0 +1,41 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Binary Package Path interface."""
+
+__metaclass__ = type
+
+__all__ = [
+    'IBinaryPackagePath',
+    'IBinaryPackagePathSource',
+    ]
+
+from zope.interface import Interface
+from zope.schema import (
+    Int,
+    TextLine,
+    )
+
+from canonical.launchpad import _
+
+
+class IBinaryPackagePath(Interface):
+    """The path of a file contained in a binary package.
+
+    A binary package release is a representation of a binary package in
+    one or more releases. This class models the files contained within
+    the binary package.
+    """
+    id = Int(title=_('ID'), required=True, readonly=True)
+    path = TextLine(title=_('Full path name'), required=True, readonly=True)
+
+
+class IBinaryPackagePathSource(Interface):
+
+    def getOrCreate(path):
+        """Fetch the ID of the given path name, or create it.
+
+        :param: path: The full path name to query or create.
+
+        :return: An ID.
+        """

=== added file 'lib/lp/soyuz/interfaces/binarypackagereleasecontents.py'
--- lib/lp/soyuz/interfaces/binarypackagereleasecontents.py	1970-01-01 00:00:00 +0000
+++ lib/lp/soyuz/interfaces/binarypackagereleasecontents.py	2011-06-19 07:26:23 +0000
@@ -0,0 +1,48 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Binary Package Release Contents interface."""
+
+__metaclass__ = type
+
+__all__ = [
+    'IBinaryPackageReleaseContents',
+    'IBinaryPackageReleaseContentsSet',
+    ]
+
+from lazr.restful.fields import Reference
+from zope.interface import Interface
+
+from canonical.launchpad import _
+from lp.soyuz.interfaces.binarypackagepath import IBinaryPackagePath
+from lp.soyuz.interfaces.binarypackagerelease import IBinaryPackageRelease
+
+
+class IBinaryPackageReleaseContents(Interface):
+    """The contents of a binary package release.
+
+    A binary package release is a representation of a binary package in
+    one or more releases. This class models the files contained within
+    the binary package.
+    """
+    binarypackagerelease = Reference(
+        IBinaryPackageRelease, title=_('Binary Package Release'),
+        required=True, readonly=True)
+    binarypackagepath = Reference(
+        IBinaryPackagePath, title=_('Binary Package Pathname'),
+        required=True, readonly=True)
+
+
+class IBinaryPackageReleaseContentsSet(Interface):
+
+    def add(bpr):
+        """Add the contents of the given binary package release.
+
+        :param: bpr: The `IBinaryPackageRelease` to add.
+        """
+
+    def remove(bpr):
+        """Remove the contents of the given binary package release.
+        
+        :param: bpr: The `IBinaryPackageRelease` to remove.
+        """

=== added file 'lib/lp/soyuz/model/binarypackagepath.py'
--- lib/lp/soyuz/model/binarypackagepath.py	1970-01-01 00:00:00 +0000
+++ lib/lp/soyuz/model/binarypackagepath.py	2011-06-19 07:26:23 +0000
@@ -0,0 +1,37 @@
+# Copyright 2009 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+__metaclass__ = type
+
+__all__ = [
+    'BinaryPackagePath',
+    ]
+
+from storm.locals import (
+    Int,
+    Storm,
+    Unicode,
+    )
+from zope.interface import implements
+
+from canonical.launchpad.interfaces.lpstorm import IMasterStore
+from lp.soyuz.interfaces.binarypackagepath import IBinaryPackagePath
+
+
+class BinaryPackagePath(Storm):
+    """See `IBinaryPackagePath`."""
+    implements(IBinaryPackagePath)
+    __storm_table__ = 'BinaryPackagePath'
+    id = Int(primary=True)
+    path = Unicode(name='path', allow_none=False)
+
+    def getOrCreate(self, path):
+        """See `IBinaryPackagePath`."""
+        store = IMasterStore(BinaryPackagePath)
+        bpp = store.find(BinaryPackagePath, BinaryPackagePath.path == path)
+        if bpp.count():
+            return bpp[0]
+        else:
+            bpp = BinaryPackagePath()
+            bpp.path = path
+            return store.add(bpp)

=== added file 'lib/lp/soyuz/model/binarypackagereleasecontents.py'
--- lib/lp/soyuz/model/binarypackagereleasecontents.py	1970-01-01 00:00:00 +0000
+++ lib/lp/soyuz/model/binarypackagereleasecontents.py	2011-06-19 07:26:23 +0000
@@ -0,0 +1,84 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+__metaclass__ = type
+
+__all__ = [
+    'BinaryPackageReleaseContents',
+    ]
+
+from fixtures import TempDir
+import os
+import subprocess
+
+from storm.locals import (
+    Int,
+    Reference,
+    Storm,
+    )
+from zope.component import getUtility
+from zope.interface import implements
+
+from canonical.launchpad.interfaces.lpstorm import IMasterStore
+from canonical.librarian.utils import copy_and_close
+from lp.soyuz.interfaces.binarypackagepath import IBinaryPackagePathSource
+from lp.soyuz.interfaces.binarypackagereleasecontents import (
+    IBinaryPackageReleaseContents,
+    )
+
+
+class BinaryPackageReleaseContents(Storm):
+    """See `IBinaryPackageReleaseContents`."""
+    implements(IBinaryPackageReleaseContents)
+    __storm_table__ = 'BinaryPackageReleaseContents'
+    __storm_primary__ = ("binarypackagerelease_id", "binaypackagepath_id")
+
+    binarypackagerelease_id = Int(
+        name='binarypackagerelease', allow_none=False)
+    binarypackagerelease = Reference(
+        binarypackagerelease_id, 'BinaryPackageRelease.id')
+
+    binaypackagepath_id = Int(name='binarypackagepath', allow_none=False)
+    binarypackagepath = Reference(
+        binaypackagepath_id, 'BinaryPackagePath.id')
+
+    def add(self, bpr):
+        """See `IBinaryPackageReleaseContents`."""
+        if not bpr.files:
+            return None
+        store = IMasterStore(BinaryPackageReleaseContents)
+        with TempDir() as tmp_dir:
+            dest = os.path.join(
+                tmp_dir.path, bpr.files[0].libraryfile.filename)
+            dest_file = open(dest, 'w')
+            bpr.files[0].libraryfile.open()
+            copy_and_close(bpr.files[0].libraryfile, dest_file)
+            process = subprocess.Popen(
+                ['dpkg-deb', '-c', dest], cwd=tmp_dir.path,
+                stdout=subprocess.PIPE)
+            contents, stderr = process.communicate()
+            ret = process.wait()
+            if ret != 0:
+                return None
+            for line in contents.split('\n'):
+                # We don't need to care about directories.
+                if line.endswith('/'):
+                    continue
+                if line == '':
+                    continue
+                split_line = line.split()
+                bpp = getUtility(IBinaryPackagePathSource).getOrCreate(
+                    unicode(split_line[-1][2:]))
+                bprc = BinaryPackageReleaseContents()
+                bprc.binarypackagerelease = bpr
+                bprc.binarypackagepath = bpp
+                store.add(bprc)
+
+    def remove(self, bpr):
+        """See `IBinaryPackageReleaseContents`."""
+        store = IMasterStore(BinaryPackageReleaseContents)
+        results = store.find(
+            BinaryPackageReleaseContents,
+            BinaryPackageReleaseContents.binarypackagerelease == bpr.id)
+        for bprc in results:
+            store.remove(bprc)

=== added file 'lib/lp/soyuz/tests/test_binarypackagepath.py'
--- lib/lp/soyuz/tests/test_binarypackagepath.py	1970-01-01 00:00:00 +0000
+++ lib/lp/soyuz/tests/test_binarypackagepath.py	2011-06-19 07:26:23 +0000
@@ -0,0 +1,27 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Test the Binary Package Path model."""
+
+__metaclass__ = type
+
+from zope.component import getUtility
+
+from canonical.testing.layers import DatabaseFunctionalLayer
+from lp.soyuz.interfaces.binarypackagepath import IBinaryPackagePathSource
+from lp.testing import TestCaseWithFactory
+
+
+class TestBinaryPackagePath(TestCaseWithFactory):
+
+    layer = DatabaseFunctionalLayer
+
+    def test_get_or_create(self):
+        bpp = getUtility(IBinaryPackagePathSource).getOrCreate(u'bin/bash')
+        self.assertEqual(u'bin/bash', bpp.path)
+
+    def test_get_or_create_existing(self):
+        orig_bpp = getUtility(IBinaryPackagePathSource).getOrCreate(
+            u'bin/bash')
+        bpp = getUtility(IBinaryPackagePathSource).getOrCreate(u'bin/bash')
+        self.assertEqual(orig_bpp.id, bpp.id)

=== added file 'lib/lp/soyuz/tests/test_binarypackagereleasecontents.py'
--- lib/lp/soyuz/tests/test_binarypackagereleasecontents.py	1970-01-01 00:00:00 +0000
+++ lib/lp/soyuz/tests/test_binarypackagereleasecontents.py	2011-06-19 07:26:23 +0000
@@ -0,0 +1,55 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Test the Binary Package Release Contents model."""
+
+__metaclass__ = type
+
+import transaction
+
+from zope.component import getUtility
+
+from canonical.launchpad.interfaces.lpstorm import IStore
+from canonical.testing.layers import LaunchpadFunctionalLayer
+from lp.archiveuploader.tests import datadir
+from lp.soyuz.interfaces.binarypackagereleasecontents import (
+    IBinaryPackageReleaseContentsSet,
+    )
+from lp.soyuz.model.binarypackagereleasecontents import (
+    BinaryPackageReleaseContents,
+    )
+from lp.testing import TestCaseWithFactory
+
+
+class TestBinaryPackagePath(TestCaseWithFactory):
+
+    layer = LaunchpadFunctionalLayer
+
+    def create_bpr(self):
+        bpr = self.factory.makeBinaryPackageRelease()
+        deb = open(datadir('pmount_0.9.7-2ubuntu2_amd64.deb'), 'r')
+        lfa = self.factory.makeLibraryFileAlias(
+            filename='pmount_0.9.7-2ubuntu2_amd64.deb', content=deb.read())
+        deb.close()
+        transaction.commit()
+        bpr.addFile(lfa)
+        return bpr
+
+    def test_add(self):
+        bpr = self.create_bpr()
+        getUtility(IBinaryPackageReleaseContentsSet).add(bpr)
+        store = IStore(BinaryPackageReleaseContents)
+        results = store.find(
+            BinaryPackageReleaseContents,
+            BinaryPackageReleaseContents.binarypackagerelease == bpr.id)
+        self.assertEqual(13, results.count())
+
+    def test_remove(self):
+        bpr = self.create_bpr()
+        getUtility(IBinaryPackageReleaseContentsSet).add(bpr)
+        getUtility(IBinaryPackageReleaseContentsSet).remove(bpr)
+        store = IStore(BinaryPackageReleaseContents)
+        results = store.find(
+            BinaryPackageReleaseContents,
+            BinaryPackageReleaseContents.binarypackagerelease == bpr.id)
+        self.assertEqual(0, results.count())

